@@ -26,15 +26,13 @@ COPY_OFF=0
 COPY_NORMAL=1
 COPY_DEEP=2
 #match operation combine
-AND=0
-OR=1
 
 PRE_ALLOC_SIZE = 300
 
 def accu_iterator(iterable, accu_method,initial_value=(None,)):
     """
     A method that enables itertools accumulation over a method
-    Note: This method is just needed because in python <3.8 itertools accumulation has no initial parameter!
+    .. note::  This method is just needed because in python <3.8 itertools accumulation has no initial parameter!
     :param iterable: iterable
     :param accu_method: accumulation method (will be fet by two parameters cumulated and new item)
     :return: accumulated iterator
@@ -59,53 +57,53 @@ def is_iterator_empty(iterator):
 
 __INTERVAL_RESULTS__ = (
     # 0 -normal range check
-    lambda v,ll,ul,lc,uc,pc: lc(v,ll) and uc(v,ul),
+    lambda v,ll,ul,lc,uc,pc,pl: lc(v,ll) and uc(v,ul),
     # 1 -not in 0
-    lambda v,ll,ul,lc,uc,pc: not (lc(v,ll) and uc(v,ul)),
+    lambda v,ll,ul,lc,uc,pc,pl: not (lc(v,ll) and uc(v,ul)),
     # 0b10~2 up_limit is None (equal)
-    lambda v,ll,ul,lc,uc,pc: v == ll ,
+    lambda v,ll,ul,lc,uc,pc,pl: v == ll ,
     # 0b11~3 up_limit is None (not equal)
-    lambda v, ll, ul, lc, uc, pc: v != ll,
+    lambda v, ll, ul, lc, uc, pc,pl: v != ll,
     # 0b100~4 low_limit is inf
-    lambda v,ll,ul,lc,uc,pc: uc(v,ul),
+    lambda v,ll,ul,lc,uc,pc,pl: uc(v,ul),
     # 0b101~5 low_limit is inf not in
-    lambda v, ll, ul, lc, uc, pc: not uc(v,ul),
+    lambda v, ll, ul, lc, uc, pc,pl: not uc(v,ul),
     # 0b110~6 up_limit is inf
-    lambda v, ll, ul, lc, uc, pc: lc(v, ll),
+    lambda v, ll, ul, lc, uc, pc,pl: lc(v, ll),
     # 0b111~7 up_limit is inf not in
-    lambda v, ll, ul, lc, uc, pc: not lc(v, ll),
+    lambda v, ll, ul, lc, uc, pc,pl: not lc(v, ll),
     # 0b1000~8 - pre and normal range check
-    lambda v, ll, ul, lc, uc, pc: pc.check(v) and lc(v, ll) and uc(v, ul),
+    lambda v, ll, ul, lc, uc, pc,pl: pc.check(v,pl) and lc(v, ll) and uc(v, ul),
     # 0b1001~9 - pre and normal range check; not in
-    lambda v, ll, ul, lc, uc, pc: not (pc.check(v) and lc(v, ll) and uc(v, ul)),
+    lambda v, ll, ul, lc, uc, pc,pl: not (pc.check(v,pl) and lc(v, ll) and uc(v, ul)),
     # 0b1010~10 - pre and up_limit is None (equal)
-    lambda v, ll, ul, lc, uc, pc: pc.check(v) and v == ll,
+    lambda v, ll, ul, lc, uc, pc,pl: pc.check(v,pl) and v == ll,
     # 0b1011~11 pre and up_limit is None (not equal)
-    lambda v, ll, ul, lc, uc, pc: pc.check(v) and v != ll,
+    lambda v, ll, ul, lc, uc, pc,pl: pc.check(v,pl) and v != ll,
     # 0b1100~12 pre and low_limit is inf
-    lambda v, ll, ul, lc, uc, pc: pc.check(v) and uc(v, ul),
+    lambda v, ll, ul, lc, uc, pc,pl: pc.check(v,pl) and uc(v, ul),
     # 0b1101~13 pre and low_limit is inf not in
-    lambda v, ll, ul, lc, uc, pc: pc.check(v) and not uc(v, ul),
+    lambda v, ll, ul, lc, uc, pc,pl: pc.check(v,pl) and not uc(v, ul),
     # 0b1110~14 pre and up_limit is inf
-    lambda v, ll, ul, lc, uc, pc: pc.check(v) and lc(v, ll),
+    lambda v, ll, ul, lc, uc, pc,pl: pc.check(v,pl) and lc(v, ll),
     # 0b1111~15 pre and up_limit is inf not in
-    lambda v, ll, ul, lc, uc, pc: pc.check(v) and not lc(v, ll),
+    lambda v, ll, ul, lc, uc, pc,pl: pc.check(v,pl) and not lc(v, ll),
     # 0b10000~8 - pre or normal range check
-    lambda v, ll, ul, lc, uc, pc: pc.check(v) or lc(v, ll) and uc(v, ul),
+    lambda v, ll, ul, lc, uc, pc,pl: pc.check(v,pl) or lc(v, ll) and uc(v, ul),
     # 0b10001~9 - pre or normal range check; not in
-    lambda v, ll, ul, lc, uc, pc: not (pc.check(v) or lc(v, ll) and uc(v, ul)),
+    lambda v, ll, ul, lc, uc, pc,pl: not (pc.check(v,pl) or lc(v, ll) and uc(v, ul)),
     # 0b10010~10 - pre or up_limit is None (equal)
-    lambda v, ll, ul, lc, uc, pc: pc.check(v) or v == ll,
+    lambda v, ll, ul, lc, uc, pc,pl: pc.check(v,pl) or v == ll,
     # 0b10011~11 pre or up_limit is None (not equal)
-    lambda v, ll, ul, lc, uc, pc: pc.check(v) or v != ll,
+    lambda v, ll, ul, lc, uc, pc,pl: pc.check(v,pl) or v != ll,
     # 0b10100~12 pre or low_limit is inf
-    lambda v, ll, ul, lc, uc, pc: pc.check(v) or uc(v, ul),
+    lambda v, ll, ul, lc, uc, pc,pl: pc.check(v,pl) or uc(v, ul),
     # 0b10101~13 pre or low_limit is inf not in
-    lambda v, ll, ul, lc, uc, pc: pc.check(v) or not uc(v, ul),
+    lambda v, ll, ul, lc, uc, pc,pl: pc.check(v,pl) or not uc(v, ul),
     # 0b10110~14 pre or up_limit is inf
-    lambda v, ll, ul, lc, uc, pc: pc.check(v) or lc(v, ll),
+    lambda v, ll, ul, lc, uc, pc,pl: pc.check(v,pl) or lc(v, ll),
     # 0b10111~15 pre or up_limit is inf not in
-    lambda v, ll, ul, lc, uc, pc: pc.check(v) or not lc(v, ll),
+    lambda v, ll, ul, lc, uc, pc,pl: pc.check(v,pl) or not lc(v, ll),
     )
 
 
@@ -115,9 +113,19 @@ class iTInterval():
 
     the class contains a check if a given value is in the defined interval or not
 
-    It's recommended to use the intervals or portion python package for data models in case more functionality is needed.
+    The class might be a little bit under estimated in all the itertree functionalities but its a short but very
+    powerful implementation of an Interval class for python.
 
-    Note:: For equal just set upper_limit to None (upper_open, lower_open parameter will be ignored in this case)
+    The class contains anything you might need in case of a Interval functionality. You can given open/closed interval
+    definitions including infinite limits. The intervals can be combined to a mathematical set via
+    the pre_interval parameter. And the check method allows to give other limits as defined.
+    This is especially useful for dynamically calculated limits.
+
+    The interval definition is also possible via a mathematical string like: "(1,2)" or "[10,+inf)".
+
+    If you need a more advanced implementation you might have a look on  the intervals/portion python package.
+
+    .. note::  For equal just set upper_limit to None (upper_open, lower_open parameter will be ignored in this case)
     """
     INF = 'inf'  # constant defining infinity limit
 
@@ -128,14 +136,12 @@ class iTInterval():
 
         the class contains a check if a given value is in the defined interval or not
 
-        Note:: For more advanced interval handling it's recommended to use the intervals or portion python package instead
-
-        Note:: For equal you give lower_limit and set upper_limit to None (lower_open,upper_open parameters will be
+        .. note::  For equal you give lower_limit and set upper_limit to None (lower_open,upper_open parameters will be
         ignored in this case). The math representation in this case is "== %s"%lower_limit
 
-        Note:: The not_in=True can be given to invert the interval check result (match is anything outside the interval)
+        .. note::  The not_in=True can be given to invert the interval check result (match is anything outside the interval)
                in the math representation we add in this case a "!" before the interval
-        Note:: Cascade interval definitions can be created the pre_interval definition
+        .. note::  Cascade interval definitions can be created the pre_interval definition
                              e.g. math_repr= "(([1,5]) and [9,12]) and [100,200]' valid values: 1...5,9...12,100..200
 
 
@@ -249,20 +255,38 @@ class iTInterval():
     def is_equal(self):
         return self.upper_limit is None
 
-    def check(self, value, lower_limit=None, upper_limit=None,return_iterator=False):
+    def check(self, value, use_limits=None,return_iterator=False):
         """
         main check function
         :param value: value to be check if in interval or not (you might give iterables too!
-        :param lower_limit: if a value is given the limit can be changed during the test (dynamic/post calculated limits)
-        :param upper_limit: if a value is given the limit can be changed during the test (dynamic/post calculated limits)
-        :return: True/False or iterator over single value check use all() to get a summary!
+        :param use_limits: You can replace the static limits in the interval with dynamic ones given in the check, any
+                           nested iterable can be used here (do not use iterators!).
+                           None - use static limit
+                           (lower,_limit, upper_limit) - replace limits in highest level interval if lower_limit or
+                                                        upper_limit is None the static one is used
+                          (((lower_limit_l2,upper_limit_l2),(lower_limit_l1,upper_limit_l1)),(lower_limit_l0,upper_limit_l0))
+                          - use nested tuples to give replacement limits to deeper levels (use None for using static ones)
+        :return: True/False or iterator over single value check use any() to get a summary!
         """
-        if lower_limit is None:
+        lower_limit = self.lower_limit
+        upper_limit = self.upper_limit
+        pre_limits=None
+        if use_limits is not None:
+            if len(use_limits)==2:
+                if use_limits[0] is not None:
+                    lower_limit = use_limits[0]
+                if use_limits[1] is not None:
+                    lower_limit = use_limits[1]
+            elif len(use_limits)==3:
+                if use_limits[1] is not None:
+                    lower_limit = use_limits[1]
+                if use_limits[2] is not None:
+                    lower_limit = use_limits[2]
+                if use_limits[0] is not None:
+                    pre_limits = use_limits[0]
+            else:
+                raise SyntaxError('Given use_limits %s not matching'%use_limits)
             # use pre defined limit
-            lower_limit = self.lower_limit
-        if upper_limit is None:
-            # use pre defined limit
-            upper_limit = self.upper_limit
         check_method=__INTERVAL_RESULTS__[self.method_key]
         if type(value) in (str,bytes):
             # we may miss here some types the generator object might be delivered in case an iterable is detected!
@@ -273,16 +297,24 @@ class iTInterval():
         elif hasattr(value, '__iter__') or hasattr(value, '__next__'):
             # iterable
             if return_iterator:
-                return_item= (check_method(v,lower_limit,upper_limit,self._low_check,self._up_check,self.pre_interval) for v in value)
+                return_item= (check_method(v,lower_limit,upper_limit,
+                                           self._low_check,self._up_check,
+                                           self.pre_interval, pre_limits) for v in value)
             else:
-                return_item= any((check_method(v, lower_limit, upper_limit, self._low_check, self._up_check, self.pre_interval)
+                return_item= any((check_method(v, lower_limit, upper_limit,
+                                               self._low_check, self._up_check,
+                                               self.pre_interval, pre_limits)
                         for v in value))
         else:
             if return_iterator:
-                return_item= iter((check_method(value, lower_limit, upper_limit, self._low_check, self._up_check, self.pre_interval),))
+                return_item= iter((check_method(value, lower_limit, upper_limit,
+                                                self._low_check, self._up_check,
+                                                self.pre_interval, pre_limits),))
 
             else:
-                return_item= check_method(value, lower_limit, upper_limit,  self._low_check,self._up_check, self.pre_interval)
+                return_item= check_method(value, lower_limit, upper_limit,
+                                          self._low_check,self._up_check,
+                                          self.pre_interval, pre_limits)
         return return_item
 
     def math_repr(self):
@@ -324,7 +356,7 @@ class iTInterval():
     def from_str(self, interval_str):
         """
         create the interval from a math representation string
-        Note:: Give inf for infinity
+        .. note::  Give inf for infinity
         :param interval_str: math string representation
         :return:
         """
@@ -473,13 +505,27 @@ class iTLink(object):
 class iTMatch(object):
     '''
     The match object is used to defined match to elements in the DtaTree used in  iterations over the DataTree
+    The defined iMatch object can be used for checks against iTree objects (mainly for checks against the tag and also
+    for string matches e.g. for finding iTree.data.keys() or .values() in filters.
     '''
 
     __slots__=('_pattern', '_op', '_check')
 
-    def __init__(self, pattern, combine=OR):
+    def __init__(self, pattern, combine_or=True):
+        """
+        Create a match pattern for different proposes. Depending on the type we have following functions:
+
+         * int     - check for an index
+         * TagIdx  - check for a TagIdx
+         * str - string pattern using fnmatch
+         * iterable like list, tuple, ... combine the given patterns with the combine key
+
+        :param pattern: give pattern
+
+        :param combine_or: True - or ; False - and; combination of matches/match patterns
+        """
         self._pattern = pattern
-        self._op = combine
+        self._op = combine_or
         self._check = self._analyse(pattern)
 
     @property
@@ -635,7 +681,7 @@ class TagIdxStr(TagIdx):
 
     Example: "mytag#1" will be translated in the TagIdx("mytag",1)
 
-    Note:: This makes only sense and can only be used if the tag is a string (not for other objects)
+    .. note::  This makes only sense and can only be used if the tag is a string (not for other objects)
 
     :param tag_idx_str: string containing the definition
 
@@ -656,7 +702,7 @@ class TagIdxBytes(TagIdxStr):
 
     Example: b"mytag#1" will be translated in the TagIdx(b"mytag",1)
 
-    Note:: This makes only sense and can only be used if the tag is a byte (not for other objects)
+    .. note::  This makes only sense and can only be used if the tag is a byte (not for other objects)
 
     :param tag_idx_bytes: bytes containing the definition
 
